#include "winver.h"

2 ICON icon/icon_jdupes_all.ico

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 1,21,2,0
 PRODUCTVERSION 1,21,2,0
 FILEFLAGSMASK 0x3fL
 FILEFLAGS 0x0L
 FILEOS 0x40004L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
		BLOCK "040904b0"
		BEGIN
		    VALUE "Comments", "(C) 2015-2023 Jody Bruchon <jody@jodybruchon.com>, published under The MIT License"
		    VALUE "CompanyName", "Jody Bruchon"
		    VALUE "FileDescription", "jdupes Duplicate File Finder Tool"
		    VALUE "FileVersion", "1,21,2,0"
		    VALUE "InternalName", "jdupes"
		    VALUE "LegalCopyright", "(C) 2015-2023 Jody Bruchon <jody@jodybruchon.com>"
		    VALUE "OriginalFilename", "jdupes.exe"
		    VALUE "ProductName", "jdupes"
		    VALUE "ProductVersion", "1,21,2,0"
		END
    END
    BLOCK "VarFileInfo"
    BEGIN
		VALUE "Translation", 0x409, 1200
    END
END
