/* VERSION determines the program's version number
 * This file is part of jdupes; see jdupes.c for license information */

#ifndef JDUPES_VERSION_H
#define JDUPES_VERSION_H

#define VER "1.21.3"
#define VERDATE "2023-02-09"

#endif /* JDUPES_VERSION_H */
