/* Jody Bruchon's sorting code library
 * See jody_sort.c for license information */

#ifndef NO_NUMSORT

#ifndef JODY_SORT_H
#define JODY_SORT_H

#ifdef __cplusplus
extern "C" {
#endif

extern int numeric_sort(char * restrict c1,
                char * restrict c2, int sort_direction);

#ifdef __cplusplus
}
#endif

#endif /* JODY_SORT_H */

#endif /* NO_NUMSORT */
